function sleep(milliseconds) {
    var start = new Date().getTime();
    for (var i = 0; i < 1e10; i++) {
        if ((new Date().getTime() - start) > milliseconds) {
            break;
        }
    }
}


function fracToSin(frac) {
    return Math.sin(frac * Math.PI / 2);
}

function hexToRgb(hex) {
    var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result ? {
        r: parseInt(result[1],16),
        g: parseInt(result[2],16),
        b: parseInt(result[3],16)
    } : null;
}
// function that takes simple modal with only text
function createSimpleModal(text) {

    const modalOverlay = document.createElement('div');
    modalOverlay.className = 'simple-modal-overlay';
    const modalContainer = document.createElement('div');
    modalContainer.className = 'simple-modal-container';
    const modalCloseRow = document.createElement('div');
    modalCloseRow.className = 'simple-modal-close-row';
    const modalCloseButton = document.createElement('div');
    modalCloseButton.className = 'simple-modal-close-button';
    modalCloseButton.textContent = 'X';

    const modalTextRow = document.createElement('div');
    modalTextRow.className = 'simple-modal-text-row';
    modalTextRow.textContent = text;
    modalCloseRow.appendChild(modalCloseButton);
    modalContainer.appendChild(modalCloseRow);
    modalContainer.appendChild(modalTextRow);
    modalOverlay.appendChild(modalContainer)
    document.body.appendChild(modalOverlay);
    popIn(modalContainer,150);
    modalCloseButton.onclick = () => {
        popOut(modalContainer,150,() => document.body.removeChild(modalOverlay));
    }
    modalOverlay.onclick = () => {
        popOut(modalContainer,150,() => document.body.removeChild(modalOverlay));
    }
    modalContainer.onclick = (e) => {
        e.stopPropagation();
    }

}

function createSimpleHTMLModal(title,text = '') {

    const modalOverlay = document.createElement('div');
    modalOverlay.className = 'simple-modal-overlay';
    const modalContainer = document.createElement('div');
    modalContainer.className = 'simple-html-modal-container';
    const modalCloseRow = document.createElement('div');
    modalCloseRow.className = 'simple-modal-close-row';
    const modalCloseButton = document.createElement('div');
    modalCloseButton.className = 'simple-modal-close-button';
    modalCloseButton.textContent = 'X';

    const modalHeaderRow = document.createElement('div');
    modalHeaderRow.className = 'simple-modal-header-row';
    modalHeaderRow.textContent = title;
    const modalTextRow = document.createElement('div');
    modalTextRow.className = 'simple-modal-text-row';
    modalTextRow.innerHTML = text;
    modalCloseRow.appendChild(modalCloseButton);
    modalContainer.appendChild(modalCloseRow);
    modalContainer.appendChild(modalHeaderRow);
    modalContainer.appendChild(modalTextRow);
    modalOverlay.appendChild(modalContainer)
    document.body.appendChild(modalOverlay);
    popIn(modalContainer,300);
    modalCloseButton.onclick = () => {
        popOut(modalContainer,150,() => document.body.removeChild(modalOverlay));
    }
    modalOverlay.onclick = () => {
        popOut(modalContainer,150,() => document.body.removeChild(modalOverlay));
    }

    document.addEventListener('keyup',(e) => {

        if (e.code === 'Escape' && modalOverlay.parentNode) {
            popOut(modalContainer,150,() => document.body.removeChild(modalOverlay));
        }
    })
    modalContainer.onclick = (e) => {
        e.stopPropagation();
    }

}


function popIn(element,duration = 1000,afterFunction) {
    element.style.transition = `scale ${duration}ms ease`;
    element.style.scale = 0;


    setTimeout(() => {
        element.style.scale = 1;
    },100);

    if (afterFunction) {
        setTimeout(() => {
            afterFunction();
        },duration + 100);
    }
}

function popOut(element,duration = 1000,afterFunction) {
    element.style.transition = `scale ${duration}ms ease`;
    element.style.scale = 1;

    setTimeout(() => {
        element.style.scale = 0;
    },100);

    if (afterFunction) {
        setTimeout(() => {
            afterFunction();
        },duration + 100)
    }
}

function returnMainRootIfEscape(e) {
    if (e.code === 'Escape') {
        returnMainRoot();
    }
}

function swapPageFromRight(html) {
    const mainRoot = document.getElementById('igem_root');
    const alterRoot = document.getElementById('igem_root_alternative');
    alterRoot.innerHTML = html;
    const duration = 500;
    document.addEventListener('keyup',returnMainRootIfEscape);
    for (let p = 1; p <= 75; p++) {
        setTimeout(() => {
            mainRoot.style.left = `-${(p / 75) * 100}vw`;
            alterRoot.style.left = `${(1 - p / 75) * 100}vw`;
            mainRoot.style.opacity = 1 - p / 75;
            alterRoot.style.opacity = p / 75;
        },duration / 75 * p);
    }

}

function returnMainRoot() {
    const mainRoot = document.getElementById('igem_root');
    const alterRoot = document.getElementById('igem_root_alternative');
    document.removeEventListener('keyup',returnMainRootIfEscape)
    const duration = 500;
    for (let p = 1; p <= 75; p++) {
        setTimeout(() => {
            mainRoot.style.left = `-${(1 - p / 75) * 100}vw`;
            alterRoot.style.left = `${(p / 75) * 100}vw`;
            alterRoot.style.opacity = 1 - p / 75;
            mainRoot.style.opacity = p / 75;
        },duration / 75 * p);
    }
    setTimeout(() => {
        alterRoot.innerHTML = '';
    },duration);

}


function loadAndSlidePage(pageName) {
    fetch(pageName).then(res => res.text().then(html => {
        swapPageFromRight(html);
    }))

}


function animateWithScroll(selector,upperBound,lowerBound) {
    const toAnimate = document.querySelectorAll(selector);
    const animate = (el) => {
        const boundRect = el.getBoundingClientRect();
        const clientY = boundRect?.y;

        if (clientY < window.innerHeight * upperBound && clientY > window.innerHeight * lowerBound) {
            const progress = Math.floor((1 - (clientY / window.innerHeight - lowerBound) / (upperBound - lowerBound)) * 100);
            el.seek(`${progress}%`)
        }

    }
    toAnimate.forEach(el => {
        animate(el);
        document.addEventListener('scroll',() => {
            animate(el);
        });

    });

}

function rotateGradient(selector,color1,color2,speed = 1) {
    const item = document.getElementById(selector);
    let deg = 90;

    const rotate = (angle) => {

        setTimeout(() => {
            item.style.background = `linear-gradient(${angle}deg, ${color1}, ${color2})`;
            item.style.backgroundClip = 'text';
            item.style.webkitBackgroundClip = 'text';
            const del = 25;
            if (angle > 90 + del && angle < 270 - del) {
                rotate((angle + 6) % 360);
            }
            else if (angle > 270 + del || angle < 90 - del) {
                rotate((angle + 6) % 360);
            }
            else {
                rotate((angle + 8) % 360);
            }

        },100 / speed);
    }
    rotate(deg);


}


function scrollFromTop(selector,duration = 1,delay = 1000) {
    const element = document.getElementById(selector);
    element.style.height = '0%';
    setTimeout(() => {
        element.style.animation = `scale_from_top ${duration}s linear`;
    },delay);
    setTimeout(() => {
        element.style.height = '100%';
    },duration * 1000 + delay);
}


function addTextOnTop(text,elementId,rootElemId = 'igem_root',params = Object) {

    const element = document.getElementById(elementId);
    const rect = element.getBoundingClientRect();
    const x = rect.x;
    const y = rect.y + window.scrollY;

    const rootElem = document.getElementById(rootElemId);
    const textElem = document.createElement('div');
    textElem.classList.add('star-above-text');
    textElem.textContent = text;
    textElem.style.fontSize = '1.8vw';
    textElem.style.top = `${y - 84 - window.innerWidth / 26 + (params.y ?? 0)}px`;

    textElem.onclick = element.onclick;

    rootElem.appendChild(textElem);

    textElem.style.left = `${x - textElem.getBoundingClientRect().width / 3 + (params.x ?? 0)}px`;


    return textElem;
}